;;;; -*- Mode: Lisp; Syntax: ANSI-Common-Lisp; Base: 10; Package: cl-user -*-
;;;; ***********************************************************************
;;;; FILE IDENTIFICATION
;;;;
;;;; Name:          utils.lisp
;;;; Version:       0.7
;;;; Project:       bosco - Cocoa application template for OpenMCL
;;;; Purpose:       common utilities for Bosco ools and apps
;;;; Programmer:    mikel evins
;;;; Date Started:  2006/11/28
;;;;
;;;; $Id: utils.lisp,v 1.2 2006/12/14 00:12:36 mikel Exp $
;;;; ***********************************************************************

(in-package :cl-user)

(eval-when (:COMPILE-TOPLEVEL :LOAD-TOPLEVEL :EXECUTE)
  (defpackage UTILITIES
	(:documentation
	 "A package for general utilities.")
	(:use common-lisp)
	(:export #:+bosco-home+
             #:ensure-directory-pathname
             #:instantiate-template
             #:instantiate-template-file
             #:path
             #:pathname-separator
             #:read-file
             #:recursive-copy-directory
             #:substitute-template-string
             #:write-file)))

;;; ----------------------------------------------------------------------
;;; string template utilities
;;; ----------------------------------------------------------------------

(defun substitute-template-string (input-text key-str val-str
                                   &optional (start 0))
  (if (>= start (length input-text))
      input-text
      (let ((pos (search key-str input-text :test #'string=)))
        (if pos
            (let ((left (subseq input-text 0 pos))
                  (right (subseq input-text (+ pos (length key-str)))))
              (substitute-template-string 
               (concatenate 'string left val-str right)
               key-str val-str (+ pos (length val-str))))
            input-text))))

(defun instantiate-template (input-template substitutions-alist)
  (if (null substitutions-alist)
      input-template
      (instantiate-template
       (substitute-template-string input-template
                                   (car (first substitutions-alist))
                                   (cdr (first substitutions-alist)))
       (cdr substitutions-alist))))

(defun instantiate-template-file (input-path output-path substitutions-alist)
  (write-file (instantiate-template (read-file input-path) substitutions-alist) output-path))

;;; ----------------------------------------------------------------------
;;; file and directory utilities
;;; ----------------------------------------------------------------------

;;; For hygiene, in case we ever want to use this code on a platform
;;; that doesn't use "/"
(defun pathname-separator ()
  #\/)

(defmethod ensure-directory-pathname ((p string))
  (let ((pstr (namestring p)))
    (if (char= (pathname-separator)
               (elt pstr (1- (length pstr))))
        p
        (pathname (concatenate 'string p (string (pathname-separator)))))))

(defmethod ensure-directory-pathname ((p pathname)) (ensure-directory-pathname (namestring p)))

(defparameter +bosco-home+ 
  ;; either the user-specified environment variable...
  (or (ccl::getenv "BOSCO_HOME")
      ;; or the current working directory when this file is loaded
      (ensure-directory-pathname (ccl::current-directory-name))))

(defun path (&rest components)
  (if (null components)
      (pathname "")
      (if (null (cdr components))
          (pathname (car components))
          (merge-pathnames (apply #'path (cdr components))
                           (ensure-directory-pathname (car components))))))

(defun recursive-copy-directory (source-path dest-path)
  (assert (not (probe-file dest-path))()
          "The file or directory '~A' already exists" dest-path)
  (ensure-directories-exist (ensure-directory-pathname dest-path))
  (let ((files (directory (path source-path "*") :directories nil :files t))
        (subdirs (directory (path source-path "*") :directories t :files nil)))
    (dolist (f files)
      (let* ((src-name (file-namestring f))
             (dest-file (path dest-path src-name)))
        (ccl:copy-file f dest-file)))
    (dolist (d subdirs)
      (let* ((subdir-name (first (last (pathname-directory d))))
             (dest-dir (ensure-directory-pathname (path dest-path subdir-name))))
        (recursive-copy-directory d dest-dir)))
    dest-path))

(defun read-file (input-path)
  (with-open-file (in-file input-path :direction :input
                            :if-does-not-exist :error)
    (let ((result (make-string (file-length in-file))))
      (read-sequence result in-file)
      result)))

(defun write-file (text output-path)
  (with-open-file (out-file output-path :direction :output
                            :if-does-not-exist :create
                            :if-exists :error)
    (write-string text out-file)))



